package cn.iocoder.yudao.module.wealth.service.receipt;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.apollo.core.event.OrderNumberLogEvent;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.code.CodeUtils;
import cn.iocoder.yudao.framework.common.util.collectors.CollectorsUtil;
import cn.iocoder.yudao.framework.dict.core.dto.DictDataRespDTO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.security.core.LoginUser;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.bpm.api.BpmCreateServiceFactory;
import cn.iocoder.yudao.module.bpm.enums.BpmProcessInstanceResultEnum;
import cn.iocoder.yudao.module.bpm.enums.WorkFlowEmus;
import cn.iocoder.yudao.module.ecw.api.currency.CurrencyApi;
import cn.iocoder.yudao.module.ecw.api.currency.dto.CurrencyRespDTO;
import cn.iocoder.yudao.module.system.api.dict.DictDataApi;
import cn.iocoder.yudao.module.wealth.convert.receipt.ReceiptConvert;
import cn.iocoder.yudao.module.wealth.convert.receiptAccount.ReceiptAccountConvert;
import cn.iocoder.yudao.module.wealth.convert.receivable.ReceivableConvert;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receipt.ReceiptDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receiptAccount.ReceiptAccountDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receiptApproval.ReceiptApprovalDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receiptItem.ReceiptItemDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receivable.ReceivableDO;
import cn.iocoder.yudao.module.wealth.dal.mysql.receipt.ReceiptMapper;
import cn.iocoder.yudao.module.wealth.dal.mysql.receiptAccount.ReceiptAccountMapper;
import cn.iocoder.yudao.module.wealth.dal.mysql.receiptItem.ReceiptItemMapper;
import cn.iocoder.yudao.module.wealth.enums.ReceiptStatusEnum;
import cn.iocoder.yudao.module.wealth.service.receiptAccount.ReceiptAccountService;
import cn.iocoder.yudao.module.wealth.service.receiptApproval.ReceiptApprovalService;
import cn.iocoder.yudao.module.wealth.service.receivable.ReceivableService;
import cn.iocoder.yudao.module.wealth.vo.receipt.*;
import cn.iocoder.yudao.module.wealth.vo.receiptAccount.ReceiptAccountCreateReqVO;
import cn.iocoder.yudao.module.wealth.vo.receivable.ReceivableUpdateReqVO;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.NumberFormat;
import java.util.*;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.wealth.enums.ErrorCodeConstants.*;

/**
 * 收款单 Service 实现类
 *
 * @author 郑屹
 */
@Service
@Validated
public class ReceiptServiceImpl extends AbstractService<ReceiptMapper, ReceiptDO> implements ReceiptService {

    @Autowired
    private ReceiptMapper receiptMapper;
    @Resource
    private ReceivableService receivableService;

    @Resource
    private ReceiptAccountService receiptAccountService;

    @Resource
    private ReceiptAccountMapper receiptAccountMapper;

    @Resource
    private ReceiptItemMapper receiptItemMapper;

    @Autowired
    private ApplicationContext applicationContext;
    @Autowired
    BpmCreateServiceFactory bpmCreateServiceFactory;

    @Autowired
    ReceiptApprovalService receiptApprovalService;

    @Resource
    DictDataApi dictDataApi;

    @Resource
    CurrencyApi currencyApi;


    private static final String RECEIPT_NEW_NUMBER = "receipt:new:number";


    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long createReceipt(ReceiptCreateReqVO createReqVO) {
        List<ReceivableUpdateReqVO> receivableUpdateReqVOList =
                createReqVO.getReceivableVOList();

        if (CollectionUtil.isEmpty(receivableUpdateReqVOList)) {
            throw exception(RECEIVABLE_NOT_NULL);
        }

        /*
         */
        if (createReqVO.getOrderId() == 0) {
            throw exception(ORDER_ID_NULL);
        }

        //校验应收明细为同一客户同一自编号同目的国下的应收明细。
        //当自编号为空（某些预付款未装柜的订单），则校验为同一客户同意目的国，忽略自编号规则，视为同一自编号
        validateSameCustomerReceivable(createReqVO.getReceivableVOList());

        List<ReceiptAccountCreateReqVO> receiptAccountCreateReqVOList =
                createReqVO.getReceiptAccountList();

        if (CollectionUtil.isEmpty(receiptAccountCreateReqVOList)) {
            throw exception(RECEIPT_ACCOUNT_NOT_EXISTS);
        }


        LoginUser user = SecurityFrameworkUtils.getLoginUser();
        String creator = String.valueOf(null == user ? "" : user.getId());
        ReceiptDO receipt = ReceiptConvert.INSTANCE.convert(createReqVO);
        //OrderNumberLogListener
        OrderNumberLogEvent event = new OrderNumberLogEvent();
        event.setType(3);
        event.setNumberKey(RECEIPT_NEW_NUMBER);
        applicationContext.publishEvent(event);
        String newReceiptNo = CodeUtils.getOddReceiptNumbers(event.getNumber());
        receipt.setReceiptNo(newReceiptNo);
        // 更新收款单号最新缓存
        // 待将最新单号持久化
        event.setNewNumber(newReceiptNo);
        applicationContext.publishEvent(event);
        receipt.setCreateTime(new Date());
        receipt.setCreator(creator);

        if (StringUtils.equals(createReqVO.getAddType(), "1")) {
            receipt.setState(ReceiptStatusEnum.DRAFT.getValue());
        }

        /*
          添加订单ID关联字段
         */
        receipt.setOrderId(createReqVO.getOrderId());
        receiptMapper.insert(receipt);

        if (StringUtils.equals(createReqVO.getAddType(), "0")) {
            validateReceiptBmpStatus(receipt.getId());
            //创建收款单并发起审核流程
            ReceiptApprovalDO approvalDO = new ReceiptApprovalDO();
            approvalDO.setReceiptId(receipt.getId());
            approvalDO.setReceiptNo(receipt.getReceiptNo());
            approvalDO.setBmpKey(WorkFlowEmus.FINANCE_RECEIPT_APPROVE.getKey());
            approvalDO.setStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
            approvalDO.setReason(createReqVO.getReason());
            receiptApprovalService.save(approvalDO);
            Long userId = SecurityFrameworkUtils.getLoginUserId();
            String bpmId = bpmCreateServiceFactory.createBmp(userId, approvalDO.getId(), WorkFlowEmus.FINANCE_RECEIPT_APPROVE.getKey(), receipt.getReceiptNo(), createReqVO.getCopyUserList());
            approvalDO.setBmpId(bpmId);
            receiptApprovalService.updateById(approvalDO);

            ReceiptDO update = new ReceiptDO();
            update.setId(receipt.getId());
            update.setBmpStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
            update.setState(ReceiptStatusEnum.APPROVE_ING.getValue());
            update.setBmpId(bpmId);
            // 添加关联的订单ID
            update.setOrderId(createReqVO.getOrderId());
            receiptMapper.updateById(update);
        }

        List<ReceivableDO> receivableDOList =
                receivableUpdateReqVOList.stream().
                        filter(re -> null != re.getId()).map(re -> {
                            ReceivableDO receivableDO =
                                    ReceivableConvert.INSTANCE.convert(re);
                            receivableDO.setReceiptNo(receipt.getReceiptNo());
                            receivableDO.setReceiptId(receipt.getId());
                            receivableDO.setState(1);
                            receivableDO.setUpdater(creator);
                            receivableDO.setUpdateTime(new Date());
                            return receivableDO;
                        }).collect(Collectors.toList());

        receivableService.updateBatchById(receivableDOList);

        List<ReceiptAccountDO> receiptAccountDOList =
                receiptAccountCreateReqVOList.stream().map(re -> {
                    ReceiptAccountDO receivableDO = ReceiptAccountConvert.INSTANCE.convert(re);
                    receivableDO.setReceiptId(receipt.getId());
                    receivableDO.setUpdater(creator);
                    receivableDO.setUpdateTime(new Date());
                    return receivableDO;
                }).collect(Collectors.toList());

        receiptAccountService.saveBatch(receiptAccountDOList);
        // 返回
        return receipt.getId();
    }

    public void validateSameCustomerReceivable(List<ReceivableUpdateReqVO> receivableUpdateReqVOList) {
        Long tempCustomerId = null;
        String tempGuojia = "";
        String tempNum = "";

        for (int i = 0; i < receivableUpdateReqVOList.size(); i++) {
            ReceivableUpdateReqVO temVo = receivableUpdateReqVOList.get(i);

            ReceivableDO receivableDO = receivableService.getReceivable(temVo.getId());

            if (null == receivableDO.getOrderId()) {
                throw exception(RECEIVABLE_ORDER_EXITS_NOT);
            }

            OrderVo orderDo = receiptMapper.getContainerNumberByOrderId(receivableDO.getOrderId());
            if (null == orderDo) {
                throw exception(RECEIVABLE_ORDER_EXITS_NOT);
            }

            if (i == 0) {
                tempCustomerId = orderDo.getCustomerId();//客户

                if (StrUtil.isNotBlank(orderDo.getContainerNumber())) {
                    tempNum = orderDo.getContainerNumber();//自编号
                }

                tempGuojia = orderDo.getCountry();
            } else {
                log.error(tempCustomerId + "");
                log.error(orderDo.getCustomerId() + "");
                if (!StringUtils.equals(tempCustomerId.toString(), orderDo.getCustomerId().toString())) {
                    throw exception(RECEIVABLE_CUSTOMER_DIFF);
                }

                if (StrUtil.isNotBlank(orderDo.getContainerNumber()) && StrUtil.isNotBlank(tempNum)) {
                    if (!StrUtil.equals(tempNum, orderDo.getContainerNumber())) {
                        throw exception(RECEIVABLE_CUSTOMER_DIFF);
                    }
                }

                if (StrUtil.isNotBlank(orderDo.getContainerNumber()) && StrUtil.isBlank(tempNum)) {
                    tempNum = orderDo.getContainerNumber();
                }

                if (!StrUtil.equals(tempGuojia, orderDo.getCountry())) {
                    throw exception(RECEIVABLE_CUSTOMER_DIFF);
                }
            }
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateReceipt(ReceiptUpdateReqVO updateReqVO) {
        //校验流程是不是正在审核中
        validateReceiptBmpStatus(updateReqVO.getId());

        List<ReceivableUpdateReqVO> receivableUpdateReqVOList = updateReqVO.getReceivableVOList();
        if (CollectionUtil.isEmpty(receivableUpdateReqVOList)) {
            throw exception(RECEIVABLE_NOT_NULL);
        }

        validateSameCustomerReceivable(updateReqVO.getReceivableVOList());


        List<ReceiptAccountCreateReqVO> receiptAccountCreateReqVOList = updateReqVO.getReceiptAccountList();
        if (CollectionUtil.isEmpty(receiptAccountCreateReqVOList)) {
            throw exception(RECEIPT_ACCOUNT_NOT_EXISTS);
        }

        LoginUser user = SecurityFrameworkUtils.getLoginUser();
        String creator = String.valueOf(null == user ? "" : user.getId());
        // 校验存在
        this.validateReceiptExists(updateReqVO.getId());
        // 更新
        ReceiptDO updateObj = ReceiptConvert.INSTANCE.convert(updateReqVO);

        if (StringUtils.equals(updateReqVO.getUpdateType(), "0")) {
            updateReqVO.setState(ReceiptStatusEnum.DRAFT.getValue());
        }

        if (StringUtils.equals(updateReqVO.getUpdateType(), "1")) {
            //更新收款单并发起审核流程
            ReceiptApprovalDO approvalDO = new ReceiptApprovalDO();
            approvalDO.setReceiptId(updateReqVO.getId());
            approvalDO.setReceiptNo(updateReqVO.getReceiptNo());
            approvalDO.setBmpKey(WorkFlowEmus.FINANCE_RECEIPT_APPROVE.getKey());
            approvalDO.setStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
            approvalDO.setReason(updateReqVO.getReason());
            receiptApprovalService.save(approvalDO);
            Long userId = SecurityFrameworkUtils.getLoginUserId();
            String bpmId = bpmCreateServiceFactory.createBmp(userId, approvalDO.getId(), WorkFlowEmus.FINANCE_RECEIPT_APPROVE.getKey(), updateReqVO.getReceiptNo(), updateReqVO.getCopyUserList());
            approvalDO.setBmpId(bpmId);
            receiptApprovalService.updateById(approvalDO);

            updateObj.setId(updateReqVO.getId());
            updateObj.setBmpStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
            updateObj.setState(ReceiptStatusEnum.APPROVE_ING.getValue());
            updateObj.setBmpId(bpmId);

        }


        receiptMapper.updateById(updateObj);

        //删除已经不存在应收
        LambdaQueryWrapper<ReceivableDO> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(ReceivableDO::getReceiptId, updateReqVO.getId());
        List<ReceivableDO> allReceivable = receivableService.list(lambdaQueryWrapper);
        for (ReceivableDO receivableDO : allReceivable) {
            if (!receivableUpdateReqVOList.stream().filter(w -> w.getId() == receivableDO.getId()).findAny().isPresent()) {
                LambdaUpdateWrapper<ReceivableDO> update = new LambdaUpdateWrapper<>();
                update.set(ReceivableDO::getReceiptId, null);
                update.set(ReceivableDO::getReceiptNo, null);
                update.eq(ReceivableDO::getId, receivableDO.getId());
                receivableService.update(update);
            }
        }


        List<ReceivableDO> receivableDOList = receivableUpdateReqVOList.stream().filter(re -> null != re.getId()).map(re -> {
            ReceivableDO receivableDO = ReceivableConvert.INSTANCE.convert(re);
            receivableDO.setReceiptNo(updateObj.getReceiptNo());
            receivableDO.setReceiptId(updateObj.getId());
            receivableDO.setState(1);
            receivableDO.setUpdater(creator);
            receivableDO.setUpdateTime(new Date());
            return receivableDO;
        }).collect(Collectors.toList());
        receivableService.updateBatchById(receivableDOList);


        LambdaQueryWrapper<ReceiptAccountDO> removeLambdaWrapper = new LambdaQueryWrapper();
        removeLambdaWrapper.eq(ReceiptAccountDO::getReceiptId, updateObj.getId());
        receiptAccountService.remove(removeLambdaWrapper);

        List<ReceiptAccountDO> receiptAccountDOList = receiptAccountCreateReqVOList.stream().map(re -> {
            ReceiptAccountDO receivableDO = ReceiptAccountConvert.INSTANCE.convert(re);
            receivableDO.setReceiptId(updateObj.getId());
            receivableDO.setUpdater(creator);
            receivableDO.setUpdateTime(new Date());
            return receivableDO;
        }).collect(Collectors.toList());

        receiptAccountService.saveBatch(receiptAccountDOList);

    }


    @Override
    public ReceiptBackVO getInvoicingInfo(Long receiptId) {
        ReceiptBackVO receiptBackVO = receiptMapper.getInvoicingInfo(receiptId);
        if (Objects.isNull(receiptBackVO)) {
            throw exception(RECEIPT_NOT_EXISTS);
        }
       /* if (receiptBackVO.getState() != 5) {
            throw exception(THIS_RECEIPT_NOT_STAY_INVOICE_STATE);
        }
        if (StringUtils.isBlank(receiptBackVO.getAccountBank()) || StringUtils.isBlank(receiptBackVO.getAccountName()) || StringUtils.isBlank(receiptBackVO.getInvoice()) || StringUtils.isBlank(receiptBackVO.getAddressPhone()) || StringUtils.isBlank(receiptBackVO.getTaxpayer())) {
            throw exception(BILLING_INFO_NOT_COMPLETE);
        }*/
        //receiptMapper.updateById(ReceiptConvert.INSTANCE.convert(receiptBackVO));
        // 根据客户ID获取用户信息
    /*    List<ReceivableDO> receivableDOList = receivableService.list(new LambdaQueryWrapper<ReceivableDO>()
                .eq(ReceivableDO::getReceiptId, receiptId)
                .eq(ReceivableDO::getDeleted, 0)
                .orderByDesc(ReceivableDO::getId));
        receiptBackVO.setReceivableList(ReceivableConvert.INSTANCE.convertList(receivableDOList));*/
        return receiptBackVO;
    }

    @Override
    public void updateInvoicingInfo(@Valid InvoicingUpdateVO invoicingUpdateVO) {
        ReceiptDO receiptDO = receiptMapper.selectById(invoicingUpdateVO.getId());
        if (Objects.isNull(receiptDO)) {
            throw exception(RECEIPT_NOT_EXISTS);
        }
        if (receiptDO.getState() != 5) {
            throw exception(THIS_RECEIPT_NOT_STAY_INVOICE_STATE);
        }
        receiptDO.setInvoicingAt(new Date());
        receiptDO.setInvoiceNumber(invoicingUpdateVO.getInvoiceNumber());
        receiptDO.setInvoicingType(invoicingUpdateVO.getInvoicingType());
        receiptDO.setInvoicingRemark(invoicingUpdateVO.getInvoicingRemark());
        receiptDO.setInvoicingTypeId(invoicingUpdateVO.getInvoicingTypeId());
        receiptDO.setTaxRate(invoicingUpdateVO.getTaxRate());
        receiptDO.setProjectName(invoicingUpdateVO.getProjectName());

        receiptDO.setInvoice(invoicingUpdateVO.getInvoice());
        receiptDO.setTaxpayer(invoicingUpdateVO.getTaxpayer());
        receiptDO.setAccountBank(invoicingUpdateVO.getAccountBank());
        receiptDO.setAccountName(invoicingUpdateVO.getAccountName());

        receiptDO.setAddressPhone(invoicingUpdateVO.getAddressPhone());

        receiptDO.setState(ReceiptStatusEnum.OPEN_BILL.getValue());
        receiptMapper.updateById(receiptDO);
    }

    @Override
    @Transactional
    public void deleteReceipt(Long id) {
        // 校验存在
        this.validateReceiptExists(id);
        // 删除
        receiptMapper.deleteById(id);
        //删除收款单的同时同步删除收款明细
        LambdaQueryWrapper<ReceiptItemDO> removeItemWrapper = new LambdaQueryWrapper<>();
        removeItemWrapper.eq(ReceiptItemDO::getReceiptId, id);
        receiptItemMapper.delete(removeItemWrapper);
        //同步更新应收明细的收款单ID为NULL
        receivableService.lambdaUpdate().eq(ReceivableDO::getReceiptId, id).set(ReceivableDO::getReceiptId, null).set(ReceivableDO::getState, 0).set(ReceivableDO::getReceiptNo, null).update();
    }

    private void validateReceiptExists(Long id) {
        if (receiptMapper.selectById(id) == null) {
            throw exception(RECEIPT_NOT_EXISTS);
        }
    }

    @Override
    public ReceiptDO getReceipt(Long id) {
        return receiptMapper.selectById(id);
    }

    @Override
    public List<ReceiptDO> getReceiptList(Collection<Long> ids) {
        return receiptMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<ReceiptBackVO> getReceiptPage(ReceiptQueryVO query,
                                                    PageVO page) {
        page.setField("er.id");
        IPage<ReceiptBackVO> mpPage = MyBatisUtils.buildPage(page);
        receiptMapper.getReceiptBackPage(mpPage, query);
        //添加明细状态计算列表
        //MxStatus
        //获取List指定字段的List
        List<Long> idList = mpPage.getRecords().stream()
                .map(ReceiptBackVO::getId)
                .collect(Collectors.toList());
        //一定要判空，不然会报错
        if(CollectionUtil.isNotEmpty(idList)) {
            //根据List获取本页中所有的明细信息
            LambdaQueryWrapper<ReceiptItemDO> lambdaQueryWrapper =
                    new LambdaQueryWrapper<>();
            lambdaQueryWrapper.in(ReceiptItemDO::getReceiptId, idList);
            //所有收款单的明细
            List<ReceiptItemDO> listItem =
                    receiptItemMapper.selectList(lambdaQueryWrapper);

            LambdaQueryWrapper<ReceiptAccountDO> lambdaQueryWrapper2 =
                    new LambdaQueryWrapper<>();
            lambdaQueryWrapper2.in(ReceiptAccountDO::getReceiptId, idList);
            List<ReceiptAccountDO> ReceiptAccountList =
                    receiptAccountMapper.selectList(lambdaQueryWrapper2);

            Map<Integer, CurrencyRespDTO> currencyMap = currencyApi.getAllCurrency();

            for (ReceiptBackVO r : mpPage.getRecords()
            ) {
                String sR = getReceiptMxStatus(listItem, r.getId());
                r.setMxStatus(sR);

                //应收
                String s1 = getYSShow(ReceiptAccountList, r.getId(), currencyMap);
                r.setYsShow(s1);
                //实收
                String s2 = getSSShow(listItem, r.getId(), currencyMap);
                r.setSsShow(s2);

                //核销比例
                r.setHxBiLi(getHxBili(ReceiptAccountList,
                        listItem, r.getId()));
            }
        }
        return PageResult.of(mpPage);
    }

    private String getPercentage(BigDecimal part, BigDecimal total) {
        BigDecimal percentage = part.divide(total, 5,
                BigDecimal.ROUND_HALF_UP).multiply(new BigDecimal("100"));
        // 输出百分比
        String sR = format(percentage) + "%";
        return sR;
    }

    private String format(BigDecimal value) {
        NumberFormat numberFormat = NumberFormat.getInstance();
        numberFormat.setMinimumFractionDigits(2);
        numberFormat.setMaximumFractionDigits(2);
        return numberFormat.format(value.doubleValue());
    }

    /*
     获取核销比例
     */
    private String getHxBili(List<ReceiptAccountDO> ReceiptAccountList,
                             List<ReceiptItemDO> listItemList,
                             long ReceiptId) {
        String sR = "0";
        if (CollectionUtil.isEmpty(ReceiptAccountList))
            return sR;
        if (CollectionUtil.isEmpty(listItemList)) {
            return sR;
        }
        List<ReceiptAccountDO> ReceiptAccountList1 =
                ReceiptAccountList.stream().filter(i -> i.getReceiptId() == ReceiptId).
                        collect(Collectors.toList());
        if (CollectionUtil.isEmpty(ReceiptAccountList1)) return sR;

        BigDecimal bTotal = new BigDecimal(0);
        BigDecimal bPart = new BigDecimal(0);
        for (ReceiptAccountDO do1 : ReceiptAccountList1
        ) {
            bTotal = bTotal.add(do1.getWriteOffAmount());
        }

        //总值为0，直接返回空
        if (bTotal.compareTo(BigDecimal.ZERO) == 0) {
            return sR;
        }

        //已核销状态记录
        List<ReceiptItemDO> list =
                listItemList.stream().filter(i -> i.getReceiptId() == ReceiptId
                        && i.getStatus() != 0).collect(Collectors.toList());
        if (CollectionUtil.isEmpty(list)) {
            //核销金额为0 核销比例为空
            return sR;
        }

        for (ReceiptItemDO do2 : list
        ) {
            bPart = bPart.add(do2.getWriteOffAmount());
        }
        //计算百分比
        sR = getPercentage(bPart, bTotal);
        return sR;
    }

    /*
     应收款
     */
    private String getYSShow(List<ReceiptAccountDO> ReceiptAccountList,
                             long ReceiptId,
                             Map<Integer, CurrencyRespDTO> currencyMap) {
        String sR = "";
        if (CollectionUtil.isEmpty(ReceiptAccountList)) {
            return sR;
        }

        List<ReceiptAccountDO> ReceiptAccountList1 =
                ReceiptAccountList.stream().filter(i -> i.getReceiptId() == ReceiptId).
                        collect(Collectors.toList());
        if (CollectionUtil.isEmpty(ReceiptAccountList1))
            return sR;

        for (ReceiptAccountDO do1 : ReceiptAccountList1
        ) {
            BigDecimal b1 = do1.getReceivableAmount();
            String s1 = b1.toString();
            String s2 = "";
            //获取币种
            if (do1.getCurrencyId() != null) {
                int n1 = Integer.parseInt(do1.getCurrencyId().toString());
                s2 = currencyMap.get(n1).getTitleZh();
            }
            sR += s1 + s2 + "\r\n";
        }
        return sR;
    }

    /*

     实收金额
     */
    private String getSSShow(List<ReceiptItemDO> listItemList,
                             long ReceiptId,
                             Map<Integer, CurrencyRespDTO> currencyMap) {
        String sR = "";
        if (CollectionUtil.isEmpty(listItemList)) {
            return sR;
        }

        List<ReceiptItemDO> list =
                listItemList.stream().filter(i -> i.getReceiptId() == ReceiptId).
                        collect(Collectors.toList());
        if (CollectionUtil.isEmpty(list)) return sR;

        //根据币种分组求和
        Map<Long, BigDecimal> dataMap =
                list.stream().filter(t -> t.getCurrencyId() != null)
                        .collect(Collectors.groupingBy(ReceiptItemDO::getCurrencyId,
                                CollectorsUtil.summingBigDecimal(ReceiptItemDO::getAmount)));
        //遍历币种信息
        for (Map.Entry<Long, BigDecimal> entry : dataMap.entrySet()) {
            String key = entry.getKey().toString();
            String s1 = entry.getValue().toString();

            String s2 = "";
            //获取币种
            int n1 = Integer.parseInt(key);
            s2 = currencyMap.get(n1).getTitleZh();
            sR += s1 + s2 + "\r\n";
        }
        return sR;
    }

    /*
      计算收款单明细状态
     收款单主表
     SELECT * from ecw_receipt where receipt_no='SKD00006547'
     收款单明细表
     SELECT * from ecw_receipt_item where receipt_id=6554
     */
    private String getReceiptMxStatus(List<ReceiptItemDO> listItem, long ReceiptId) {
        //未录入 收款单下无收款明细 OK
        //未提交 ：收款单下有收款明细且全未提交 OK
        //部分提交  收款单下有收款明细且部分提交(无需判断收款明细是否审批) OK
        //未审核  收款明细全部提交且全部未审批  OK
        //部分审核  收款明细全部提交且部分已审批  OK
        //全部审核  收款明细全部已审批  OK
        String sMxStatus = "";
        //计算明细总数
        Long iCount = listItem.stream().filter(i -> i.getReceiptId() == ReceiptId).
                collect(Collectors.counting());
        if (iCount == 0) {
            sMxStatus = "未录入";
            return sMxStatus;
        } else {
            //存在记录就判断状态
            //status 状态0待核销，待提交 1已核销
            //2核销审核中，3反核销审核中
            //bmpStatus
            //PROCESS(1, "处理中"),APPROVE(2, "通过"),
            //REJECT(3, "不通过"),CANCEL(4, "已取消");
            long Item0 = listItem.stream().filter(i -> i.getStatus() == 0).
                    filter(i -> i.getReceiptId() == ReceiptId).
                    collect(Collectors.counting());
            long Item1 = listItem.stream().filter(i -> i.getStatus() == 1).
                    filter(i -> i.getReceiptId() == ReceiptId).
                    collect(Collectors.counting());
            long Item2 = listItem.stream().filter(i -> i.getStatus() == 2).
                    filter(i -> i.getReceiptId() == ReceiptId).
                    collect(Collectors.counting());
            long Item3 = listItem.stream().filter(i -> i.getStatus() == 3).
                    filter(i -> i.getReceiptId() == ReceiptId).
                    collect(Collectors.counting());

            if (Item0 == iCount) {
                //0待核销
                //收款单下有收款明细且全未提交
                sMxStatus = "未提交";
                return sMxStatus;
            }

            if (Item0 > 0 && Item0 < iCount) {
                //部分提交  收款单下有收款明细且部分提交(无需判断收款明细是否审批)
                sMxStatus = "部分提交";
                return sMxStatus;
            }

            //进部分提交，部分审核逻辑
            //未审核  收款明细全部提交且全部未审批
            if (Item2 == iCount) {
                sMxStatus = "未审核";
                return sMxStatus;
            }

            if (Item1 == iCount) {
                //1已核销
                sMxStatus = "全部审核";
                return sMxStatus;
            }

            if ((Item2 + Item3) == iCount ||
                    (Item1 + Item3) == iCount ||
                    (Item2 + Item1) == iCount) {
                sMxStatus = "部分审核";
                return sMxStatus;
            }
        }
        return sMxStatus;
    }

    @Override
    public List<ReceiptDO> getReceiptList(ReceiptQueryVO query) {
        return receiptMapper.selectList(query);
    }

    @Override
    public void verification(Long id) {
        ReceiptDO receiptDO = receiptMapper.selectById(id);
        if (receiptDO == null) {
            throw exception(RECEIPTITEM_NOT_EXISTS);
        }
        if (receiptDO.getState() == null || receiptDO.getState() == 1) {
            ReceiptDO updateReceiptDO = new ReceiptDO();
            updateReceiptDO.setId(receiptDO.getId());
            if (receiptDO.getOpenInvoice() == 0) {
                updateReceiptDO.setState(2);
            }
            if (receiptDO.getOpenInvoice() == 1) {
                updateReceiptDO.setState(3);
            }
            receiptMapper.updateById(updateReceiptDO);
        } else {
            throw exception(RECEIPT_VERIFICATION);
        }
    }

    @Override
    public void verificationCancel(Long id) {
        ReceiptDO receiptDO = receiptMapper.selectById(id);
        if (receiptDO == null) {
            throw exception(RECEIPTITEM_NOT_EXISTS);
        }
        if (receiptDO.getState() == null || receiptDO.getState() == 1) {
            throw exception(RECEIPT_VERIFICATION_NOT);
        } else {
            ReceiptDO updateReceiptDO = new ReceiptDO();
            updateReceiptDO.setId(receiptDO.getId());
            updateReceiptDO.setState(1);
            receiptMapper.updateById(updateReceiptDO);
        }

    }

    @Override
    public void createFinanceApproveNo(FinanceReceiptVo financeReceiptVo) {
        validateReceiptBmpStatus(financeReceiptVo.getReceiptId());
        //创建收款单反审核流程
        ReceiptApprovalDO approvalDO = new ReceiptApprovalDO();
        approvalDO.setReceiptId(financeReceiptVo.getReceiptId());
        approvalDO.setReceiptNo(financeReceiptVo.getReceiptNo());
        approvalDO.setBmpKey(WorkFlowEmus.FINANCE_RECEIPT_APPROVE_NO.getKey());
        approvalDO.setStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
        approvalDO.setReason(financeReceiptVo.getRemark());
        receiptApprovalService.save(approvalDO);
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        String bpmId = bpmCreateServiceFactory.createBmp(userId, approvalDO.getId(), WorkFlowEmus.FINANCE_RECEIPT_APPROVE_NO.getKey(), financeReceiptVo.getReceiptNo(), financeReceiptVo.getCopyUserList());
        approvalDO.setBmpId(bpmId);
        receiptApprovalService.updateById(approvalDO);

        ReceiptDO update = new ReceiptDO();
        update.setId(financeReceiptVo.getReceiptId());
        update.setBmpStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
        update.setState(ReceiptStatusEnum.APPROVE_NEGATION.getValue());
        update.setBmpId(bpmId);
        receiptMapper.updateById(update);
    }

    @Override
    public void createFinanceReceiptWriteOff(FinanceReceiptVo financeReceiptVo) {

        ReceiptDO receiptDO =
                validateReceiptBmpStatus(financeReceiptVo.getReceiptId());
        //判断收款单下的收款明细是不是全部核销完成
        LambdaQueryWrapper<ReceiptItemDO> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(ReceiptItemDO::getReceiptId, financeReceiptVo.getReceiptId());
        lambdaQueryWrapper.in(ReceiptItemDO::getStatus, 0, 2, 3);
        Long count = receiptItemMapper.selectCount(lambdaQueryWrapper);
        if (count != 0) {
            throw exception(RECEIPT_ITEM_WRITE_OFF_ING);
        }

        LambdaQueryWrapper<ReceiptItemDO> lambdaQueryWrapper1 =
                new LambdaQueryWrapper<>();
        lambdaQueryWrapper1.eq(ReceiptItemDO::getReceiptId,
                financeReceiptVo.getReceiptId());
        lambdaQueryWrapper1.eq(ReceiptItemDO::getStatus, 1);
        List<ReceiptItemDO> receiptItemDOList =
                receiptItemMapper.selectList(lambdaQueryWrapper1);
        if (null == receiptItemDOList || receiptItemDOList.size() == 0) {
            throw exception(RECEIPT_NOT_WRITE_OFF_ITEM_NOT_EXISTS);
        }

        BigDecimal totalAmount =
                receiptItemDOList.stream().map(ReceiptItemDO::getWriteOffAmount).reduce(BigDecimal.ZERO, BigDecimal::add);


       /* if (receiptDO.getState() != 2 && receiptDO.getState() != 3) {
            throw exception(RECEIPT_WRITE_OFF_STATUS_NOT);
        }*/


        String financeRemark = "";
        Long currencyId = receiptDO.getCurrencyId();

        if (null == currencyId) {
            throw exception(RECEIPT_CURRENCY_NOT_EXISTS);
        }
        if (totalAmount.compareTo(receiptDO.getReceivableTotalAmount()) == -1) {
            DictDataRespDTO dictDataRespDTO = dictDataApi.parseDictDataFromCache("write_off_diff", String.valueOf(currencyId));
            if (null == dictDataRespDTO) {
                throw exception(RECEIPT_CURRENCY_DIFF_NOT_EXISTS);
            }
            //核销差额大于配置提示
            if (receiptDO.getReceivableTotalAmount().subtract(totalAmount).compareTo(new BigDecimal(dictDataRespDTO.getValue())) == 1) {
                throw exception(RECEIPT_CURRENCY_DIFF_BIG);
            } else {
                String message = "收款单已全部核销，总金额差值小于{}{},允许全部核销，核销比例应为100%";
                Map<Integer, CurrencyRespDTO> currencyMap = currencyApi.getAllCurrency();
                CurrencyRespDTO currencyRespDTO = currencyMap.get(currencyId.intValue());
                financeRemark = StrUtil.format(message, dictDataRespDTO.getValue(), currencyRespDTO.getFuhao());
            }
        }

        //创建收款单核销流程
        ReceiptApprovalDO approvalDO = new ReceiptApprovalDO();
        approvalDO.setReceiptId(financeReceiptVo.getReceiptId());
        approvalDO.setReceiptNo(financeReceiptVo.getReceiptNo());
        approvalDO.setBmpKey(WorkFlowEmus.FINANCE_RECEIPT_WRITE_OFF.getKey());
        approvalDO.setStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
        approvalDO.setReason(financeReceiptVo.getRemark());
        receiptApprovalService.save(approvalDO);
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        String bpmId = bpmCreateServiceFactory.createBmp(userId,
                approvalDO.getId(),
                WorkFlowEmus.FINANCE_RECEIPT_WRITE_OFF.getKey(),
                financeReceiptVo.getReceiptNo(),
                financeReceiptVo.getCopyUserList());

        approvalDO.setBmpId(bpmId);
        receiptApprovalService.updateById(approvalDO);

        ReceiptDO update = new ReceiptDO();
        update.setId(financeReceiptVo.getReceiptId());
        update.setBmpStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
        //计算出核销比例
        BigDecimal proportion =
                totalAmount.divide(receiptDO.getReceivableTotalAmount(),
                        8, RoundingMode.HALF_UP).multiply(new BigDecimal("100")).setScale(6, BigDecimal.ROUND_HALF_UP);
        update.setWriteOffProportion(proportion);
        update.setState(ReceiptStatusEnum.WRITE_OFF_ALL_ING.getValue());
        update.setBmpId(bpmId);
        update.setFinanceRemark(financeRemark);
        receiptMapper.updateById(update);
    }

    @Override
    public void createFinanceReceiptWriteOffNo(FinanceReceiptVo financeReceiptVo) {
        validateReceiptBmpStatus(financeReceiptVo.getReceiptId());
        //创建收款单反核销流程
        ReceiptApprovalDO approvalDO = new ReceiptApprovalDO();
        approvalDO.setReceiptId(financeReceiptVo.getReceiptId());
        approvalDO.setReceiptNo(financeReceiptVo.getReceiptNo());
        approvalDO.setBmpKey(WorkFlowEmus.FINANCE_RECEIPT_WRITE_OFF_NO.getKey());
        approvalDO.setStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
        approvalDO.setReason(financeReceiptVo.getRemark());
        receiptApprovalService.save(approvalDO);
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        String bpmId = bpmCreateServiceFactory.createBmp(userId, approvalDO.getId(), WorkFlowEmus.FINANCE_RECEIPT_WRITE_OFF_NO.getKey(), financeReceiptVo.getReceiptNo(), financeReceiptVo.getCopyUserList());
        approvalDO.setBmpId(bpmId);
        receiptApprovalService.updateById(approvalDO);

        ReceiptDO update = new ReceiptDO();
        update.setId(financeReceiptVo.getReceiptId());
        update.setBmpStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
        update.setState(ReceiptStatusEnum.WRITE_OFF_ALL_NEGATION.getValue());
        update.setBmpId(bpmId);
        receiptMapper.updateById(update);
    }

    @Override
    public void cancelFinanceReceiptApproval(FinanceReceiptCancelVo financeReceiptCancelVo) {
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        ReceiptDO receiptDO = validateReceiptBmpCancel(financeReceiptCancelVo.getReceiptId());
        bpmCreateServiceFactory.cancelBmpById(userId, receiptDO.getBmpId(), financeReceiptCancelVo.getRemark());
    }

    @Override
    public void cancelFinanceReceiptApprovalNo(FinanceReceiptCancelVo financeReceiptCancelVo) {
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        ReceiptDO receiptDO = validateReceiptBmpCancel(financeReceiptCancelVo.getReceiptId());
        bpmCreateServiceFactory.cancelBmpById(userId, receiptDO.getBmpId(), financeReceiptCancelVo.getRemark());
    }

    @Override
    public void cancelFinanceReceiptWriteOff(FinanceReceiptCancelVo financeReceiptCancelVo) {
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        ReceiptDO receiptDO = validateReceiptBmpCancel(financeReceiptCancelVo.getReceiptId());
        bpmCreateServiceFactory.cancelBmpById(userId, receiptDO.getBmpId(), financeReceiptCancelVo.getRemark());
    }

    @Override
    public void cancelFinanceReceiptWriteOffNo(FinanceReceiptCancelVo financeReceiptCancelVo) {
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        ReceiptDO receiptDO = validateReceiptBmpCancel(financeReceiptCancelVo.getReceiptId());
        bpmCreateServiceFactory.cancelBmpById(userId, receiptDO.getBmpId(), financeReceiptCancelVo.getRemark());
    }

    private ReceiptDO validateReceiptBmpStatus(Long id) {
        ReceiptDO receiptDO = receiptMapper.selectById(id);
        if (null == receiptDO) {
            throw exception(RECEIPT_NOT_EXISTS);
        }
        if (null != receiptDO.getBmpStatus() && receiptDO.getBmpStatus() == BpmProcessInstanceResultEnum.PROCESS.getResult()) {
            throw exception(RECEIPT_APPROVING);
        }

        if (receiptDO.getState() == ReceiptStatusEnum.WRITE_OFF_ALL_ING.getValue()) {
            throw exception(RECEIPT_APPROVING);
        }
        return receiptDO;
    }

    private ReceiptDO validateReceiptBmpCancel(Long id) {
        ReceiptDO receiptDO = receiptMapper.selectById(id);
        if (null == receiptDO) {
            throw exception(RECEIPT_NOT_EXISTS);
        }
        if (null == receiptDO.getBmpStatus()) {

            throw exception(BMP_NOT);
        }

        if (receiptDO.getBmpStatus() !=
                BpmProcessInstanceResultEnum.PROCESS.getResult()) {
            throw exception(BMP_CANCEL);
        }

        return receiptDO;
    }

    @Override
    public ReceiptExportDto getReceiptExportByOrderId(Long orderId) {
        return receiptMapper.getReceiptExportByOrderId(orderId);
    }
}
